@@ -1,10 +1,10 @@
 createAquifer.base <-
-function(name,area,label,capacity,rechargeTS,Sy,leakageFraction,initialStorage,leakageCode,priority)
+function(name,area,label,volume,rechargeTS,Sy,leakageFraction,initialStorage,leakageCode,priority)
 {
    aquifer<-list(name           =name           ,
                  area           =area           ,
                  label          =label          ,
-                 capacity       =capacity       ,
+                 volume         =volume         ,
                  rechargeTS     =rechargeTS     ,
                  Sy             =Sy             ,
                  leakageFraction=leakageFraction,
